function wc = wcRjl(X,L,varargin)

% Wishart based change detection in time series of k polarimetric SAR images
%
% wc = wcRjl(cat(4,L62,L63,L64,L65,L68,L74),13,'full');
% wc = wcRjl('dir',13,'full');
%
% wcRjl in the first example runs wishart_change_Rj for the combinations
%   cat(4,L62,L63,L64,L65,L68,L74)  - result in wc(1)
%   cat(4,L63,L64,L65,L68,L74)      - result in wc(2)
%   cat(4,L64,L65,L68,L74)          - ...
%   cat(4,L65,L68,L74)
%   cat(4,L68,L74)
% and outputs the result to structure wc as wc(1), wc(2), ..., wc(5)
%
% In the second example wcRjl reads all files associated with files named
% '*.hdr' in directory 'dir' (for example '.'),
% concatenates them as in the first example
% followed by the same analysis (file names must be such that the
% concatenation happens in the right temporal order)
%
% Input
%   X       - directory name (in which case we read all files associated
%             with files named '*.hdr')
%             or
%             series of k 2x2 or 3x3 complex covariance matrices
%             (polarimetric SAR images in covariance matrix form,
%             reading order is alphabetical (as read by freadenvisar):
%               hhhh (real), hhhv (complex), hhvv (complex),
%                            hvhv (real),    hvvv (complex),
%                                            vvvv (real)
%   L       - k (or one) number of looks for X
%   pol     - 'full', 'azim', 'dual', 'fdiag' or 'ddiag'
%             (needed only when confusion is possible)
%
% Output
%   wc      - structure with
%             lnR     - logarithm of test quantities Q and Rj
%                       ln(Q) is lnR(:,:,1), ln(R_2) is lnR(:,:,2) etc.
%             f       - number of degrees of freedom for Q and Rj test
%                       statistics
%             rho
%             omega2
%             P       - probabilities of finding smaller value of
%                       -2*rho(i)*lnR(:,:,i), i=1,...,k
%             sumMat  -
%             logDet  -
%
% Reference
%
% Knut Conradsen, Allan Aasbjerg Nielsen and Henning Skriver (2016):
% "Determining the Points of Change in Time Series of Polarimetric SAR Data".
% IEEE Transactions on Geoscience and Remote Sensing 54(5), 3007-3024.
% http://www.imm.dtu.dk/pubdb/p.php?6825
% http://orbit.dtu.dk/en/publications/determining-the-points-of-change-in-time-series-of-polarimetric-sar-data(4255ddb4-1fd9-4d66-9aad-e8049a976d75).html

% (c) Copyright 2014-2017
% Allan Aasbjerg Nielsen, PhD
% alan@dtu.dk, http://people.compute.dtu.dk/alan
% 29 Oct 2014 - 11 Aug 2017

% % output 8-byte wc with ln Q/Rj, and sumMat/logDet depending on options
% out4byte = true;
% to save space, output 4-byte wc with no ln Q/Rj and no sumMat/logDet
out4byte = true;

if nargin<2 || nargin>4
    help wcRjl
    error('wcRjl: wrong input');
end
if nargout>1
    help wcRjl
    error('wcRjl: wrong output');
end
outputMat = true;
if nargin>3 % output sumMat and logDet?
    outputMat = varargin{2};
end
if nargin>4 % time points for each acquisition
    error('wcRjl: more input arguments not implemented yet')
end

if ischar(X) % read all files in directory 'X' associated with file names '*.hdr'
    directory = X;
    F = dir(strcat(directory,'/*.hdr'));
    for ii=1:length(F)
        fname = strcat(directory,'/',F(ii).name(1:end-4));
        fprintf('wcRjl: file %s\n',fname)
        X(:,:,:,ii) = freadenvisar(fname,'b');
    end
%     F.name
    clear F ii fname directory Y
end

pX = size(X);
% nrows = pX(1);
% ncols = pX(2);
nv = pX(3);
nsets = pX(4);
if nsets<2
    error('wcRjl: one set only? possibly wrong polarization')
end
% k = nsets;
% % fprintf('%s\n', '*** order of input images is ***very*** important ***')

if nv==9
    pol = 'full';
elseif nv==5
    pol = 'azim';
elseif nv==4
    pol = 'dual';
elseif nv==3
    pol = 'fdiag';
elseif nv==2
    pol = 'ddiag';
else
    pol = 'single';
%     error('wcRjl: input must contain a multplicative of 2, 3, 4, 5 or 9 variables');
end
if nargin>2
    pol = varargin{1};
    if strncmp(pol,'s',1)
        nv = 1;
    elseif strncmp(pol,'dd',2)
        nv = 2;
    elseif strncmp(pol,'fd',2)
        nv = 3;
    elseif strncmp(pol,'d',1)
        nv = 4;
    elseif strncmp(pol,'a',1)
        nv = 5;
    elseif strncmp(pol,'f',1)
        nv = 9;
    end
%     if rem(nvars,nv)~=0
%         error('wcRjl: number of input frames and polarization do not match')
%     end
end

if out4byte
% to save space, output 4-byte, no ln Q/Rj, no sumMat/logDet
    for i=1:(nsets-1)
    %     if outputMat
    %         [wc(i).lnR,wc(i).f,wc(i).rho,wc(i).omega2,wc(i).P,wc(i).sumMat,wc(i).logDet] = ...
    %             wishart_change_Rj(X(:,:,:,i:nsets),L,pol);
    %     else
            [~,wcif,wcirho,wciomega2,wciP] = ...
                wishart_change_Rj(X(:,:,:,i:nsets),L,pol);
            wc(i).f = single(wcif);
            wc(i).rho = single(wcirho);
            wc(i).omega2 = single(wciomega2);
            wc(i).P = single(wciP);
            clear wciP
    %     end
        if exist('F.name','var')
            wc(i).fname = {F(i).name(1:end-4); F(i+1).name(1:end-4)};
        end
    end
    % if outputMat
    %     wcnew = struct('lnR',[],'f',nan,'rho',nan,'omega2',nan,'P',[],...
    %         'sumMat',L*X(:,:,:,nsets),...
    %         'logDet',log(wishart_det(L*X(:,:,:,nsets))));
    %     wc = [wc,wcnew];
    %     clear wcnew
    % end
else % output 8-byte
    for i=1:(nsets-1)
        if outputMat
            [wc(i).lnR,wc(i).f,wc(i).rho,wc(i).omega2,wc(i).P,wc(i).sumMat,wc(i).logDet] = ...
                wishart_change_Rj(X(:,:,:,i:nsets),L,pol);
        else
            [wc(i).lnR,wc(i).f,wc(i).rho,wc(i).omega2,wc(i).P] = ...
                wishart_change_Rj(X(:,:,:,i:nsets),L,pol);
        end
        if exist('F.name','var')
            wc(i).fname = {F(i).name(1:end-4); F(i+1).name(1:end-4)};
        end
    end
    if outputMat
        wcnew = struct('lnR',[],'f',nan,'rho',nan,'omega2',nan,'P',[],...
            'sumMat',L*X(:,:,:,nsets),...
            'logDet',log(wishart_det(L*X(:,:,:,nsets))));
        wc = [wc,wcnew];
        clear wcnew
    end
end
